@@ -1,12 +1,15 @@
 # ACTION_CHECKBOX_NAME is unused, but should stay since its import from here
 # has been referenced in documentation.
 from django.contrib.admin.decorators import register
+from django.contrib.admin.filters import (
+    AllValuesFieldListFilter, BooleanFieldListFilter, ChoicesFieldListFilter,
+    DateFieldListFilter, FieldListFilter, ListFilter, RelatedFieldListFilter,
+    RelatedOnlyFieldListFilter, SimpleListFilter,
+)
 from django.contrib.admin.helpers import ACTION_CHECKBOX_NAME
-from django.contrib.admin.options import (HORIZONTAL, VERTICAL,
-    ModelAdmin, StackedInline, TabularInline)
-from django.contrib.admin.filters import (ListFilter, SimpleListFilter,
-    FieldListFilter, BooleanFieldListFilter, RelatedFieldListFilter,
-    ChoicesFieldListFilter, DateFieldListFilter, AllValuesFieldListFilter)
+from django.contrib.admin.options import (
+    HORIZONTAL, VERTICAL, ModelAdmin, StackedInline, TabularInline,
+)
 from django.contrib.admin.sites import AdminSite, site
 from django.utils.module_loading import autodiscover_modules
 
@@ -15,7 +18,7 @@ __all__ = [
     "StackedInline", "TabularInline", "AdminSite", "site", "ListFilter",
     "SimpleListFilter", "FieldListFilter", "BooleanFieldListFilter",
     "RelatedFieldListFilter", "ChoicesFieldListFilter", "DateFieldListFilter",
-    "AllValuesFieldListFilter", "autodiscover",
+    "AllValuesFieldListFilter", "RelatedOnlyFieldListFilter", "autodiscover",
 ]
 
 
