@@ -1,21 +1,20 @@
-# -*- coding: utf-8 -*-
-from __future__ import unicode_literals
-
+import errno
 import gzip
 import os
 import struct
 import tempfile
 import unittest
-from io import BytesIO, StringIO
+from io import BytesIO, StringIO, TextIOWrapper
+from unittest import mock
 
 from django.core.files import File
 from django.core.files.base import ContentFile
 from django.core.files.move import file_move_safe
 from django.core.files.temp import NamedTemporaryFile
-from django.core.files.uploadedfile import SimpleUploadedFile, UploadedFile
-from django.test import mock
-from django.utils import six
-from django.utils._os import upath
+from django.core.files.uploadedfile import (
+    InMemoryUploadedFile, SimpleUploadedFile, TemporaryUploadedFile,
+    UploadedFile,
+)
 
 try:
     from PIL import Image
@@ -43,6 +42,23 @@ class FileTests(unittest.TestCase):
         self.assertTrue(f.closed)
         self.assertTrue(orig_file.closed)
 
+    def test_open_resets_opened_file_to_start_and_returns_context_manager(self):
+        file = File(BytesIO(b'content'))
+        file.read()
+        with file.open() as f:
+            self.assertEqual(f.read(), b'content')
+
+    def test_open_reopens_closed_file_and_returns_context_manager(self):
+        temporary_file = tempfile.NamedTemporaryFile(delete=False)
+        file = File(temporary_file)
+        try:
+            file.close()
+            with file.open() as f:
+                self.assertFalse(f.closed)
+        finally:
+            # remove temporary file
+            os.unlink(file.name)
+
     def test_namedtemporaryfile_closes(self):
         """
         The symbol django.core.files.NamedTemporaryFile is assigned as
@@ -120,18 +136,36 @@ class FileTests(unittest.TestCase):
         f = File(StringIO('one\ntwo\nthree'))
         self.assertEqual(list(f), ['one\n', 'two\n', 'three'])
 
+    def test_readable(self):
+        with tempfile.TemporaryFile() as temp, File(temp, name='something.txt') as test_file:
+            self.assertTrue(test_file.readable())
+        self.assertFalse(test_file.readable())
+
+    def test_writable(self):
+        with tempfile.TemporaryFile() as temp, File(temp, name='something.txt') as test_file:
+            self.assertTrue(test_file.writable())
+        self.assertFalse(test_file.writable())
+        with tempfile.TemporaryFile('rb') as temp, File(temp, name='something.txt') as test_file:
+            self.assertFalse(test_file.writable())
+
     def test_seekable(self):
-        """
-        File.seekable() should be available on Python 3.
-        """
-        with tempfile.TemporaryFile() as temp:
-            temp.write(b"contents\n")
-            test_file = File(temp, name="something.txt")
-            if six.PY2:
-                self.assertFalse(hasattr(test_file, 'seekable'))
-            if six.PY3:
-                self.assertTrue(hasattr(test_file, 'seekable'))
-                self.assertTrue(test_file.seekable())
+        with tempfile.TemporaryFile() as temp, File(temp, name='something.txt') as test_file:
+            self.assertTrue(test_file.seekable())
+        self.assertFalse(test_file.seekable())
+
+    def test_io_wrapper(self):
+        content = "vive l'été\n"
+        with tempfile.TemporaryFile() as temp, File(temp, name='something.txt') as test_file:
+            test_file.write(content.encode())
+            test_file.seek(0)
+            wrapper = TextIOWrapper(test_file, 'utf-8', newline='\n')
+            self.assertEqual(wrapper.read(), content)
+            wrapper.write(content)
+            wrapper.seek(0)
+            self.assertEqual(wrapper.read(), content * 2)
+            test_file = wrapper.detach()
+            test_file.seek(0)
+            self.assertEqual(test_file.read(), (content * 2).encode())
 
 
 class NoNameFileTestCase(unittest.TestCase):
@@ -152,26 +186,55 @@ class ContentFileTestCase(unittest.TestCase):
 
     def test_content_file_custom_name(self):
         """
-        Test that the constructor of ContentFile accepts 'name' (#16590).
+        The constructor of ContentFile accepts 'name' (#16590).
         """
         name = "I can have a name too!"
         self.assertEqual(ContentFile(b"content", name=name).name, name)
 
     def test_content_file_input_type(self):
         """
-        Test that ContentFile can accept both bytes and unicode and that the
-        retrieved content is of the same type.
+        ContentFile can accept both bytes and strings and the retrieved content
+        is of the same type.
         """
         self.assertIsInstance(ContentFile(b"content").read(), bytes)
-        if six.PY3:
-            self.assertIsInstance(ContentFile("español").read(), six.text_type)
-        else:
-            self.assertIsInstance(ContentFile("español").read(), bytes)
+        self.assertIsInstance(ContentFile("español").read(), str)
+
+    def test_open_resets_file_to_start_and_returns_context_manager(self):
+        file = ContentFile(b'content')
+        with file.open() as f:
+            self.assertEqual(f.read(), b'content')
+        with file.open() as f:
+            self.assertEqual(f.read(), b'content')
+
+    def test_size_changing_after_writing(self):
+        """ContentFile.size changes after a write()."""
+        f = ContentFile('')
+        self.assertEqual(f.size, 0)
+        f.write('Test ')
+        f.write('string')
+        self.assertEqual(f.size, 11)
+        with f.open() as fh:
+            self.assertEqual(fh.read(), 'Test string')
+
+
+class InMemoryUploadedFileTests(unittest.TestCase):
+    def test_open_resets_file_to_start_and_returns_context_manager(self):
+        uf = InMemoryUploadedFile(StringIO('1'), '', 'test', 'text/plain', 1, 'utf8')
+        uf.read()
+        with uf.open() as f:
+            self.assertEqual(f.read(), '1')
+
+
+class TemporaryUploadedFileTests(unittest.TestCase):
+    def test_extension_kept(self):
+        """The temporary file name has the same suffix as the original file."""
+        with TemporaryUploadedFile('test.txt', 'text/plain', 1, 'utf8') as temp_file:
+            self.assertTrue(temp_file.file.name.endswith('.upload.txt'))
 
 
 class DimensionClosingBug(unittest.TestCase):
     """
-    Test that get_image_dimensions() properly closes files (#8817)
+    get_image_dimensions() properly closes files (#8817)
     """
     @unittest.skipUnless(Image, "Pillow not installed")
     def test_not_closing_of_files(self):
@@ -191,11 +254,11 @@ class DimensionClosingBug(unittest.TestCase):
         """
         # We need to inject a modified open() builtin into the images module
         # that checks if the file was closed properly if the function is
-        # called with a filename instead of an file object.
+        # called with a filename instead of a file object.
         # get_image_dimensions will call our catching_open instead of the
         # regular builtin one.
 
-        class FileWrapper(object):
+        class FileWrapper:
             _closed = []
 
             def __init__(self, f):
@@ -213,7 +276,7 @@ class DimensionClosingBug(unittest.TestCase):
 
         images.open = catching_open
         try:
-            images.get_image_dimensions(os.path.join(os.path.dirname(upath(__file__)), "test1.png"))
+            images.get_image_dimensions(os.path.join(os.path.dirname(__file__), "test1.png"))
         finally:
             del images.open
         self.assertTrue(FileWrapper._closed)
@@ -221,7 +284,7 @@ class DimensionClosingBug(unittest.TestCase):
 
 class InconsistentGetImageDimensionsBug(unittest.TestCase):
     """
-    Test that get_image_dimensions() works properly after various calls
+    get_image_dimensions() works properly after various calls
     using a file handler (#11158)
     """
     @unittest.skipUnless(Image, "Pillow not installed")
@@ -229,7 +292,7 @@ class InconsistentGetImageDimensionsBug(unittest.TestCase):
         """
         Multiple calls of get_image_dimensions() should return the same size.
         """
-        img_path = os.path.join(os.path.dirname(upath(__file__)), "test.png")
+        img_path = os.path.join(os.path.dirname(__file__), "test.png")
         with open(img_path, 'rb') as fh:
             image = images.ImageFile(fh)
             image_pil = Image.open(fh)
@@ -244,7 +307,7 @@ class InconsistentGetImageDimensionsBug(unittest.TestCase):
         Regression test for #19457
         get_image_dimensions fails on some pngs, while Image.size is working good on them
         """
-        img_path = os.path.join(os.path.dirname(upath(__file__)), "magic.png")
+        img_path = os.path.join(os.path.dirname(__file__), "magic.png")
         size = images.get_image_dimensions(img_path)
         with open(img_path, 'rb') as fh:
             self.assertEqual(size, Image.open(fh).size)
@@ -261,7 +324,7 @@ class GetImageDimensionsTests(unittest.TestCase):
         brokenimg.png is not a valid image and it has been generated by:
         $ echo "123" > brokenimg.png
         """
-        img_path = os.path.join(os.path.dirname(upath(__file__)), "brokenimg.png")
+        img_path = os.path.join(os.path.dirname(__file__), "brokenimg.png")
         with open(img_path, 'rb') as fh:
             size = images.get_image_dimensions(fh)
             self.assertEqual(size, (None, None))
@@ -274,12 +337,18 @@ class GetImageDimensionsTests(unittest.TestCase):
         Emulates the Parser feed error. Since the error is raised on every feed
         attempt, the resulting image size should be invalid: (None, None).
         """
-        img_path = os.path.join(os.path.dirname(upath(__file__)), "test.png")
+        img_path = os.path.join(os.path.dirname(__file__), "test.png")
         with mock.patch('PIL.ImageFile.Parser.feed', side_effect=struct.error):
             with open(img_path, 'rb') as fh:
                 size = images.get_image_dimensions(fh)
                 self.assertEqual(size, (None, None))
 
+    def test_webp(self):
+        img_path = os.path.join(os.path.dirname(__file__), 'test.webp')
+        with open(img_path, 'rb') as fh:
+            size = images.get_image_dimensions(fh)
+        self.assertEqual(size, (540, 405))
+
 
 class FileMoveSafeTests(unittest.TestCase):
     def test_file_move_overwrite(self):
@@ -296,6 +365,30 @@ class FileMoveSafeTests(unittest.TestCase):
         os.close(handle_a)
         os.close(handle_b)
 
+    def test_file_move_copystat_cifs(self):
+        """
+        file_move_safe() ignores a copystat() EPERM PermissionError. This
+        happens when the destination filesystem is CIFS, for example.
+        """
+        copystat_EACCES_error = PermissionError(errno.EACCES, 'msg')
+        copystat_EPERM_error = PermissionError(errno.EPERM, 'msg')
+        handle_a, self.file_a = tempfile.mkstemp()
+        handle_b, self.file_b = tempfile.mkstemp()
+        try:
+            # This exception is required to reach the copystat() call in
+            # file_safe_move().
+            with mock.patch('django.core.files.move.os.rename', side_effect=OSError()):
+                # An error besides EPERM isn't ignored.
+                with mock.patch('django.core.files.move.copystat', side_effect=copystat_EACCES_error):
+                    with self.assertRaises(PermissionError):
+                        file_move_safe(self.file_a, self.file_b, allow_overwrite=True)
+                # EPERM is ignored.
+                with mock.patch('django.core.files.move.copystat', side_effect=copystat_EPERM_error):
+                    self.assertIsNone(file_move_safe(self.file_a, self.file_b, allow_overwrite=True))
+        finally:
+            os.close(handle_a)
+            os.close(handle_b)
+
 
 class SpooledTempTests(unittest.TestCase):
     def test_in_memory_spooled_temp(self):
