@@ -1,8 +1,7 @@
-from __future__ import unicode_literals
-
 from datetime import datetime
 
 from django.test import TestCase
+from django.utils.deprecation import RemovedInDjango30Warning
 
 from .models import Article, IndexErrorArticle, Person
 
@@ -10,10 +9,13 @@ from .models import Article, IndexErrorArticle, Person
 class EarliestOrLatestTests(TestCase):
     """Tests for the earliest() and latest() objects methods"""
 
+    @classmethod
+    def setUpClass(cls):
+        super().setUpClass()
+        cls._article_get_latest_by = Article._meta.get_latest_by
+
     def tearDown(self):
-        """Makes sure Article has a get_latest_by"""
-        if not Article._meta.get_latest_by:
-            Article._meta.get_latest_by = 'pub_date'
+        Article._meta.get_latest_by = self._article_get_latest_by
 
     def test_earliest(self):
         # Because no Articles exist yet, earliest() raises ArticleDoesNotExist.
@@ -28,11 +30,11 @@ class EarliestOrLatestTests(TestCase):
             headline="Article 2", pub_date=datetime(2005, 7, 27),
             expire_date=datetime(2005, 7, 28)
         )
-        Article.objects.create(
+        a3 = Article.objects.create(
             headline="Article 3", pub_date=datetime(2005, 7, 28),
             expire_date=datetime(2005, 8, 27)
         )
-        Article.objects.create(
+        a4 = Article.objects.create(
             headline="Article 4", pub_date=datetime(2005, 7, 28),
             expire_date=datetime(2005, 7, 30)
         )
@@ -59,12 +61,32 @@ class EarliestOrLatestTests(TestCase):
         # in the Model.Meta
         Article.objects.model._meta.get_latest_by = None
         with self.assertRaisesMessage(
-            AssertionError,
-            "earliest() and latest() require either a field_name parameter or "
-            "'get_latest_by' in the model"
+            ValueError,
+            "earliest() and latest() require either fields as positional "
+            "arguments or 'get_latest_by' in the model's Meta."
         ):
             Article.objects.earliest()
 
+        # Earliest publication date, earliest expire date.
+        self.assertEqual(
+            Article.objects.filter(pub_date=datetime(2005, 7, 28)).earliest('pub_date', 'expire_date'),
+            a4,
+        )
+        # Earliest publication date, latest expire date.
+        self.assertEqual(
+            Article.objects.filter(pub_date=datetime(2005, 7, 28)).earliest('pub_date', '-expire_date'),
+            a3,
+        )
+
+        # Meta.get_latest_by may be a tuple.
+        Article.objects.model._meta.get_latest_by = ('pub_date', 'expire_date')
+        self.assertEqual(Article.objects.filter(pub_date=datetime(2005, 7, 28)).earliest(), a4)
+
+    def test_earliest_fields_and_field_name(self):
+        msg = 'Cannot use both positional arguments and the field_name keyword argument.'
+        with self.assertRaisesMessage(ValueError, msg):
+            Article.objects.earliest('pub_date', field_name='expire_date')
+
     def test_latest(self):
         # Because no Articles exist yet, latest() raises ArticleDoesNotExist.
         with self.assertRaises(Article.DoesNotExist):
@@ -74,7 +96,7 @@ class EarliestOrLatestTests(TestCase):
             headline="Article 1", pub_date=datetime(2005, 7, 26),
             expire_date=datetime(2005, 9, 1)
         )
-        Article.objects.create(
+        a2 = Article.objects.create(
             headline="Article 2", pub_date=datetime(2005, 7, 27),
             expire_date=datetime(2005, 7, 28)
         )
@@ -109,21 +131,51 @@ class EarliestOrLatestTests(TestCase):
         # Error is raised if get_latest_by isn't in Model.Meta.
         Article.objects.model._meta.get_latest_by = None
         with self.assertRaisesMessage(
-            AssertionError,
-            "earliest() and latest() require either a field_name parameter or "
-            "'get_latest_by' in the model"
+            ValueError,
+            "earliest() and latest() require either fields as positional "
+            "arguments or 'get_latest_by' in the model's Meta."
         ):
             Article.objects.latest()
 
+        # Latest publication date, latest expire date.
+        self.assertEqual(Article.objects.filter(pub_date=datetime(2005, 7, 27)).latest('pub_date', 'expire_date'), a3)
+        # Latest publication date, earliest expire date.
+        self.assertEqual(
+            Article.objects.filter(pub_date=datetime(2005, 7, 27)).latest('pub_date', '-expire_date'),
+            a2,
+        )
+
+        # Meta.get_latest_by may be a tuple.
+        Article.objects.model._meta.get_latest_by = ('pub_date', 'expire_date')
+        self.assertEqual(Article.objects.filter(pub_date=datetime(2005, 7, 27)).latest(), a3)
+
+    def test_latest_fields_and_field_name(self):
+        msg = 'Cannot use both positional arguments and the field_name keyword argument.'
+        with self.assertRaisesMessage(ValueError, msg):
+            Article.objects.latest('pub_date', field_name='expire_date')
+
     def test_latest_manual(self):
         # You can still use latest() with a model that doesn't have
         # "get_latest_by" set -- just pass in the field name manually.
         Person.objects.create(name="Ralph", birthday=datetime(1950, 1, 1))
         p2 = Person.objects.create(name="Stephanie", birthday=datetime(1960, 2, 3))
-        with self.assertRaises(AssertionError):
+        msg = (
+            "earliest() and latest() require either fields as positional arguments "
+            "or 'get_latest_by' in the model's Meta."
+        )
+        with self.assertRaisesMessage(ValueError, msg):
             Person.objects.latest()
         self.assertEqual(Person.objects.latest("birthday"), p2)
 
+    def test_field_name_kwarg_deprecation(self):
+        Person.objects.create(name='Deprecator', birthday=datetime(1950, 1, 1))
+        msg = (
+            'The field_name keyword argument to earliest() and latest() '
+            'is deprecated in favor of passing positional arguments.'
+        )
+        with self.assertWarnsMessage(RemovedInDjango30Warning, msg):
+            Person.objects.latest(field_name='birthday')
+
 
 class TestFirstLast(TestCase):
 
@@ -153,7 +205,7 @@ class TestFirstLast(TestCase):
             # We know that we've broken the __iter__ method, so the queryset
             # should always raise an exception.
             with self.assertRaises(IndexError):
-                IndexErrorArticle.objects.all()[0]
+                IndexErrorArticle.objects.all()[:10:2]
             with self.assertRaises(IndexError):
                 IndexErrorArticle.objects.all().first()
             with self.assertRaises(IndexError):
